@@ -8,10 +8,10 @@ from enum import Enum
 from typing import Any, Type, Union, cast, overload
 from uuid import UUID
 
-from pytket.backends.status import WAITING_STATUS, StatusEnum
+import httpx
 from quantinuum_schemas.models.backend_config import config_name_to_class
 from quantinuum_schemas.models.hypertket_config import HyperTketConfig
-from websockets.client import connect
+from websockets.asyncio.client import connect, process_exception
 from websockets.exceptions import ConnectionClosed
 
 import qnexus.exceptions as qnx_exc
@@ -30,7 +30,6 @@ from qnexus.models.annotations import Annotations, PropertiesDict
 from qnexus.models.filters import (
     CreatorFilter,
     FuzzyNameFilter,
-    JobStatusEnum,
     JobStatusFilter,
     JobTypeFilter,
     PaginationFilter,
@@ -42,7 +41,7 @@ from qnexus.models.filters import (
     SortFilterEnum,
     TimeFilter,
 )
-from qnexus.models.job_status import JobStatus
+from qnexus.models.job_status import WAITING_STATUS, JobStatus, JobStatusEnum
 from qnexus.models.language import Language
 from qnexus.models.references import (
     CircuitRef,
@@ -53,9 +52,12 @@ from qnexus.models.references import (
     ExecutionProgram,
     ExecutionResult,
     ExecutionResultRef,
+    GpuDecoderConfigRef,
+    IncompleteJobItemRef,
     JobRef,
     JobType,
     ProjectRef,
+    SystemRef,
     WasmModuleRef,
 )
 from qnexus.models.utils import assert_never
@@ -159,6 +161,7 @@ def _to_jobref(data: dict[str, Any]) -> DataframableList[CompileJobRef | Execute
 
     for entry in data["data"]:
         project_id = entry["relationships"]["project"]["data"]["id"]
+
         project_details = next(
             proj for proj in data["included"] if proj["id"] == project_id
         )
@@ -168,6 +171,27 @@ def _to_jobref(data: dict[str, Any]) -> DataframableList[CompileJobRef | Execute
             contents_modified=project_details["attributes"]["contents_modified"],
             archived=project_details["attributes"]["archived"],
         )
+
+        system_id: str | None = (
+            entry["relationships"]["system"]["data"]["id"]
+            if "system" in entry["relationships"]
+            else None
+        )
+        system_details = (
+            next(item for item in data["included"] if item["id"] == system_id)
+            if system_id is not None
+            else None
+        )
+
+        system = (
+            SystemRef(
+                id=UUID(system_id),
+                name=system_details["attributes"]["name"],
+                provider_name=system_details["attributes"]["provider_name"],
+            )
+            if system_id is not None and system_details is not None
+            else None
+        )
         job_type: Type[CompileJobRef] | Type[ExecuteJobRef]
         match entry["attributes"]["job_type"]:
             case JobType.COMPILE:
@@ -184,7 +208,9 @@ def _to_jobref(data: dict[str, Any]) -> DataframableList[CompileJobRef | Execute
                 job_type=entry["attributes"]["job_type"],
                 last_status=JobStatus.from_dict(entry["attributes"]["status"]).status,
                 last_message=JobStatus.from_dict(entry["attributes"]["status"]).message,
+                last_status_detail=JobStatus.from_dict(entry["attributes"]["status"]),
                 project=project,
+                system=system,
             )
         )
     return DataframableList(job_list)
@@ -254,6 +280,29 @@ def _fetch_by_id(job_id: UUID | str, scope: ScopeFilterEnum | None) -> JobRef:
         contents_modified=project_details["attributes"]["contents_modified"],
         archived=project_details["attributes"]["archived"],
     )
+
+    system_id: str | None = (
+        job_data["data"]["relationships"]["system"]["data"]["id"]
+        if "system" in job_data["data"]["relationships"]
+        else None
+    )
+
+    system_details = (
+        next(item for item in job_data["included"] if item["id"] == system_id)
+        if system_id is not None
+        else None
+    )
+
+    system = (
+        SystemRef(
+            id=UUID(system_id),
+            name=system_details["attributes"]["name"],
+            provider_name=system_details["attributes"]["provider_name"],
+        )
+        if system_id is not None and system_details is not None
+        else None
+    )
+
     job_type: Type[CompileJobRef] | Type[ExecuteJobRef]
     match job_data["data"]["attributes"]["job_type"]:
         case JobType.COMPILE:
@@ -279,14 +328,18 @@ def _fetch_by_id(job_id: UUID | str, scope: ScopeFilterEnum | None) -> JobRef:
         last_message=JobStatus.from_dict(
             job_data["data"]["attributes"]["status"]
         ).message,
+        last_status_detail=JobStatus.from_dict(
+            job_data["data"]["attributes"]["status"]
+        ),
         project=project,
         backend_config_store=backend_config,
+        system=system,
     )
 
 
 def wait_for(
     job: JobRef,
-    wait_for_status: StatusEnum = StatusEnum.COMPLETED,
+    wait_for_status: JobStatusEnum = JobStatusEnum.COMPLETED,
     timeout: float | None = 900.0,
 ) -> JobStatus:
     """Check job status until the job is complete (or a specified status)."""
@@ -297,8 +350,27 @@ def wait_for(
         )
     )
 
-    if job_status.status == StatusEnum.ERROR:
+    if (
+        job_status.status == JobStatusEnum.ERROR
+        and wait_for_status != JobStatusEnum.ERROR
+    ):
         raise qnx_exc.JobError(f"Job errored with detail: {job_status.error_detail}")
+    if (
+        job_status.status == JobStatusEnum.CANCELLED
+        and wait_for_status != JobStatusEnum.CANCELLED
+    ):
+        raise qnx_exc.JobError("Job was cancelled")
+    if (
+        job_status.status == JobStatusEnum.DEPLETED
+        and wait_for_status != JobStatusEnum.DEPLETED
+    ):
+        raise qnx_exc.JobError("Job has run out of account credits")
+    if (
+        job_status.status == JobStatusEnum.TERMINATED
+        and wait_for_status != JobStatusEnum.TERMINATED
+    ):
+        raise qnx_exc.JobError("Job has been terminated")
+
     return job_status
 
 
@@ -315,7 +387,7 @@ def status(job: JobRef) -> JobStatus:
 
 
 async def listen_job_status(
-    job: JobRef, wait_for_status: StatusEnum = StatusEnum.COMPLETED
+    job: JobRef, wait_for_status: JobStatusEnum = JobStatusEnum.COMPLETED
 ) -> JobStatus:
     """Check the Status of a Job via a websocket connection.
     Will use SSO tokens."""
@@ -324,22 +396,28 @@ async def listen_job_status(
     if job_status.status not in WAITING_STATUS or job_status.status == wait_for_status:
         return job_status
 
-    # If we pass True into the websocket connection, it sets a default SSLContext.
-    # See: https://websockets.readthedocs.io/en/stable/reference/client.html
-    ssl_reconfigured: Union[bool, ssl.SSLContext] = True
-    if not CONFIG.httpx_verify:
-        ssl_reconfigured = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
-        ssl_reconfigured.check_hostname = False
-        ssl_reconfigured.verify_mode = ssl.CERT_NONE
+    ssl_context = httpx.create_ssl_context(verify=CONFIG.httpx_verify)
 
-    extra_headers = {
+    def _process_exception(exc: Exception) -> Exception | None:
+        """Utility wrapper around process_exception that tells the websockets
+        library not to auto-retry SSLErrors as they are usually not recoverable.
+
+        Unfortunately SSLError inherits from OSError which websockets will always
+        retried when `connect` is used in an async for loop.
+        """
+        if isinstance(exc, ssl.SSLError):
+            return exc
+        return process_exception(exc)
+
+    additional_headers = {
         # TODO, this cookie will expire frequently
         "Cookie": f"myqos_id={get_nexus_client().auth.cookies.get('myqos_id')}"  # type: ignore
     }
     async for websocket in connect(
         f"{CONFIG.websockets_url}/api/jobs/v1beta3/{job.id}/attributes/status/ws",
-        ssl=ssl_reconfigured,
-        extra_headers=extra_headers,
+        ssl=ssl_context,
+        additional_headers=additional_headers,
+        process_exception=_process_exception,
         # logger=logger,
     ):
         try:
@@ -370,19 +448,22 @@ async def listen_job_status(
 @overload
 def results(
     job: CompileJobRef, allow_incomplete: bool = False
-) -> DataframableList[CompilationResultRef]: ...
+) -> DataframableList[CompilationResultRef | IncompleteJobItemRef]: ...
 
 
 @overload
 def results(
     job: ExecuteJobRef, allow_incomplete: bool = False
-) -> DataframableList[ExecutionResultRef]: ...
+) -> DataframableList[ExecutionResultRef | IncompleteJobItemRef]: ...
 
 
 def results(
     job: CompileJobRef | ExecuteJobRef,
     allow_incomplete: bool = False,
-) -> DataframableList[CompilationResultRef] | DataframableList[ExecutionResultRef]:
+) -> (
+    DataframableList[CompilationResultRef | IncompleteJobItemRef]
+    | DataframableList[ExecutionResultRef | IncompleteJobItemRef]
+):
     """Get the ResultRefs from a JobRef, if the job is complete.
     To enable fetching results from Jobs with incomplete items, set allow_incomplete=True.
     """
@@ -397,7 +478,7 @@ def results(
 
 def retry_submission(
     job: JobRef,
-    retry_status: list[StatusEnum] | None = None,
+    retry_status: list[JobStatusEnum] | None = None,
     remote_retry_strategy: RemoteRetryStrategy = RemoteRetryStrategy.DEFAULT,
     user_group: str | None = None,
 ) -> None:
@@ -484,9 +565,18 @@ def compile(
 
     compile_results = results(compile_job_ref)
 
-    return DataframableList(
-        [compile_result.get_output() for compile_result in compile_results]
-    )
+    compiled_circuits: list[CircuitRef] = []
+    for compile_result in compile_results:
+        if isinstance(compile_result, CompilationResultRef):
+            compiled_circuits.append(compile_result.get_output())
+        elif isinstance(compile_result, IncompleteJobItemRef):
+            raise qnx_exc.ResourceFetchFailed(
+                f"Compile job item {compile_result.job_item_integer_id} is in status {compile_result.last_status}"
+            )
+        else:
+            assert_never(compile_result)
+
+    return DataframableList(compiled_circuits)
 
 
 @accept_circuits_for_programs
@@ -500,11 +590,9 @@ def execute(
     properties: PropertiesDict | None = None,
     project: ProjectRef | None = None,
     valid_check: bool = True,
-    postprocess: bool = False,
-    noisy_simulator: bool = True,
     wasm_module: WasmModuleRef | None = None,
+    gpu_decoder_config: GpuDecoderConfigRef | None = None,
     language: Language = Language.AUTO,
-    seed: int | None = None,
     credential_name: str | None = None,
     user_group: str | None = None,
     timeout: float | None = 300.0,
@@ -525,11 +613,9 @@ def execute(
         properties=properties,
         project=project,
         valid_check=valid_check,
-        postprocess=postprocess,
-        noisy_simulator=noisy_simulator,
         wasm_module=wasm_module,
+        gpu_decoder_config=gpu_decoder_config,
         language=language,
-        seed=seed,
         credential_name=credential_name,
         user_group=user_group,
     )
@@ -538,4 +624,28 @@ def execute(
 
     execute_results = results(execute_job_ref)
 
-    return [result.download_result() for result in execute_results]
+    ex_results: list[ExecutionResult] = []
+    for result in execute_results:
+        if isinstance(result, ExecutionResultRef):
+            ex_results.append(result.download_result())
+        elif isinstance(result, IncompleteJobItemRef):
+            raise qnx_exc.ResourceFetchFailed(
+                f"Compile job item {result.job_item_integer_id} is in status {result.last_status}"
+            )
+        else:
+            assert_never(result)
+
+    return ex_results
+
+
+def cost(job: CompileJobRef | ExecuteJobRef) -> float:
+    """Get the HQC cost of a job from a JobRef."""
+    resp = get_nexus_client().get(f"/api/jobs/v1beta3/{job.id}")
+    if resp.status_code != 200:
+        raise qnx_exc.ResourceFetchFailed(
+            message=resp.text, status_code=resp.status_code
+        )
+    resp_data = resp.json()["data"]
+    job_status = resp_data["attributes"]["status"]
+    cost = job_status.get("cost")
+    return float(cost) if cost is not None else 0.0
