@@ -26,12 +26,13 @@ from hugr.qsystem.result import QsysResult
 from pydantic import BaseModel, ConfigDict, Field, field_serializer
 from pytket.backends.backendinfo import BackendInfo
 from pytket.backends.backendresult import BackendResult
-from pytket.backends.status import StatusEnum
 from pytket.circuit import Circuit
 from pytket.wasm.wasm import WasmModuleHandler
 from quantinuum_schemas.models.backend_config import BackendConfig
 
+from qnexus.exceptions import IncompatibleResultVersion
 from qnexus.models.annotations import Annotations
+from qnexus.models.job_status import JobStatus, JobStatusEnum
 from qnexus.models.utils import assert_never
 
 
@@ -139,6 +140,26 @@ class ProjectRef(BaseRef):
         )
 
 
+class SystemRef(BaseRef):
+    """Proxy object to a System in Nexus."""
+
+    id: UUID
+    name: str
+    provider_name: str
+    type: Literal["SystemRef"] = "SystemRef"
+
+    def df(self) -> pd.DataFrame:
+        """Present in a pandas DataFrame."""
+        return pd.DataFrame(
+            {
+                "id": self.id,
+                "name": self.name,
+                "provider_name": self.provider_name,
+            },
+            index=[0],
+        )
+
+
 class CircuitRef(BaseRef):
     """Proxy object to a Circuit in Nexus."""
 
@@ -202,6 +223,40 @@ class WasmModuleRef(BaseRef):
         )
 
 
+class GpuDecoderConfigRef(BaseRef):
+    """Proxy object to a GpuDecoderConfig in Nexus."""
+
+    annotations: Annotations
+    project: ProjectRef
+    id: UUID
+    _contents: str | None = None
+    type: Literal["GpuDecoderConfigRef"] = "GpuDecoderConfigRef"
+
+    def download_gpu_decoder_config_contents(self) -> str:
+        """Get the contents of the original uploaded gpu decoder config."""
+        if self._contents:
+            return self._contents
+
+        from qnexus.client.gpu_decoder_configs import (
+            _fetch_gpu_decoder_config,
+        )
+
+        self._contents = _fetch_gpu_decoder_config(self)
+        return self._contents
+
+    def df(self) -> pd.DataFrame:
+        """Present in a pandas DataFrame."""
+        return self.annotations.df().join(
+            pd.DataFrame(
+                {
+                    "project": self.project.annotations.name,
+                    "id": self.id,
+                },
+                index=[0],
+            )
+        )
+
+
 class HUGRRef(BaseRef):
     """Proxy object to a HUGR in Nexus."""
 
@@ -209,6 +264,7 @@ class HUGRRef(BaseRef):
     project: ProjectRef
     id: UUID
     _contents: Package | None = None
+    _bytes: bytes | None = None
     type: Literal["HUGRRef"] = "HUGRRef"
 
     def download_hugr(self) -> Package:
@@ -222,6 +278,17 @@ class HUGRRef(BaseRef):
         self._contents = _fetch_hugr_package(self)
         return self._contents
 
+    def download_hugr_bytes(self) -> bytes:
+        """Get the HUGR bytes of the original uploaded HUGR."""
+
+        if self._bytes:
+            return self._bytes
+
+        from qnexus.client.hugr import _fetch_hugr_bytes
+
+        self._bytes = _fetch_hugr_bytes(self)
+        return self._bytes
+
     def df(self) -> pd.DataFrame:
         """Present in a pandas DataFrame."""
         return self.annotations.df().join(
@@ -282,9 +349,11 @@ class JobRef(BaseRef):
 
     annotations: Annotations
     job_type: JobType
-    last_status: StatusEnum
+    last_status: JobStatusEnum
     last_message: str
+    last_status_detail: JobStatus | None = None
     project: ProjectRef
+    system: SystemRef | None = None
     id: UUID
     backend_config_store: BackendConfig | None = None
     type: Literal["JobRef", "CompileJobRef", "ExecuteJobRef"] = "JobRef"
@@ -310,6 +379,10 @@ class JobRef(BaseRef):
                     "last_status": self.last_status,
                     "project": self.project.annotations.name,
                     "backend_config": self.backend_config.__class__.__name__,
+                    "system": self.system.name if self.system else "Unknown",
+                    "cost": self.last_status_detail.cost
+                    if self.last_status_detail
+                    else "Unknown",
                     "id": self.id,
                 },
                 index=[0],
@@ -340,10 +413,13 @@ class CompilationResultRef(BaseRef):
 
     annotations: Annotations
     project: ProjectRef
+    last_status_detail: JobStatus | None = None
     _input_circuit: CircuitRef | None = None
     _output_circuit: CircuitRef | None = None
     _compilation_passes: DataframableList[CompilationPassRef] | None = None
     id: UUID  # compilation id
+    job_item_id: UUID | None = None
+    job_item_integer_id: int | None = None
     type: Literal["CompilationResultRef"] = "CompilationResultRef"
 
     def get_input(self) -> CircuitRef:
@@ -390,6 +466,8 @@ class CompilationResultRef(BaseRef):
                 {
                     "project": self.project.annotations.name,
                     "id": self.id,
+                    "job_item_id": self.job_item_id,
+                    "job_item_integer_id": self.job_item_integer_id,
                 },
                 index=[0],
             )
@@ -403,8 +481,22 @@ class ResultType(str, Enum):
     QSYS = "QSYS"
 
 
+class ResultVersions(int, Enum):
+    """Enumerate the valid values for requesting results in a specific format"""
+
+    DEFAULT = 3
+    RAW = 4
+
+
+class QIRResult:
+    results: str
+
+    def __init__(self, results: str):
+        self.results = results
+
+
 ExecutionProgram: TypeAlias = CircuitRef | HUGRRef | QIRRef
-ExecutionResult: TypeAlias = QsysResult | BackendResult
+ExecutionResult: TypeAlias = QsysResult | BackendResult | QIRResult
 
 
 class ExecutionResultRef(BaseRef):
@@ -413,10 +505,15 @@ class ExecutionResultRef(BaseRef):
     annotations: Annotations
     project: ProjectRef
     result_type: ResultType = ResultType.PYTKET
+    cost: float | None = None
+    last_status_detail: JobStatus | None = None
     _input_program: ExecutionProgram | None = None
     _result: ExecutionResult | None = None
+    _result_version: ResultVersions | None = None
     _backend_info: BackendInfo | None = None
     id: UUID
+    job_item_id: UUID | None = None
+    job_item_integer_id: int | None = None
     type: Literal["ExecutionResultRef"] = "ExecutionResultRef"
 
     def get_input(self) -> ExecutionProgram:
@@ -428,19 +525,23 @@ class ExecutionResultRef(BaseRef):
             self._result,
             self._backend_info,
             self._input_program,
-        ) = self._get_execute_results()
+        ) = self._get_execute_results(ResultVersions.DEFAULT)
+        self._result_version = ResultVersions.DEFAULT
         return copy(self._input_program)
 
-    def download_result(self) -> ExecutionResult:
+    def download_result(
+        self, version: ResultVersions = ResultVersions.DEFAULT
+    ) -> ExecutionResult:
         """Get a copy of the result of the program execution."""
-        if self._result:
+        if self._result and self._result_version == version:
             return copy(self._result)
 
         (
             self._result,
             self._backend_info,
             self._input_program,
-        ) = self._get_execute_results()
+        ) = self._get_execute_results(version=version)
+        self._result_version = version
         return copy(self._result)
 
     def download_backend_info(self) -> BackendInfo:
@@ -452,13 +553,20 @@ class ExecutionResultRef(BaseRef):
             self._result,
             self._backend_info,
             self._input_program,
-        ) = self._get_execute_results()
+        ) = self._get_execute_results(ResultVersions.DEFAULT)
+        self._result_version = ResultVersions.DEFAULT
         return copy(self._backend_info)
 
     def _get_execute_results(
-        self,
+        self, version: ResultVersions
     ) -> tuple[ExecutionResult, BackendInfo, ExecutionProgram]:
-        """Utility method to retrieve the passes and output circuit."""
+        """Utility method to retrieve the passes and output circuit.
+        result_version can be passed to request v4 results for qsys results only.
+
+        Default results for any program type on H series devices is pytket style results
+        Default result for QIR programs on NG devices is QIR standard compliant results.
+        Default result for HUGR programs is NG results.
+        """
         from qnexus.client.jobs._execute import (
             _fetch_pytket_execution_result,
             _fetch_qsys_execution_result,
@@ -466,9 +574,13 @@ class ExecutionResultRef(BaseRef):
 
         match self.result_type:
             case ResultType.PYTKET:
+                if version != ResultVersions.DEFAULT:
+                    raise IncompatibleResultVersion(
+                        "pytket results can only be fetched in the default version"
+                    )
                 return _fetch_pytket_execution_result(self)
             case ResultType.QSYS:
-                return _fetch_qsys_execution_result(self)
+                return _fetch_qsys_execution_result(self, version)
             case _:
                 assert_never(self.result_type)
 
@@ -480,6 +592,39 @@ class ExecutionResultRef(BaseRef):
                     "project": self.project.annotations.name,
                     "id": self.id,
                     "result_type": self.result_type,
+                    "cost": self.cost,
+                    "job_item_id": self.job_item_id,
+                    "job_item_integer_id": self.job_item_integer_id,
+                },
+                index=[0],
+            )
+        )
+
+
+class IncompleteJobItemRef(BaseRef):
+    """Proxy object to a Job Item in Nexus that is not complete."""
+
+    annotations: Annotations
+    id: UUID = UUID(int=0)  # Incomplete items have no result ID
+    job_item_id: UUID | None = None
+    job_item_integer_id: int | None = None
+    project: ProjectRef
+    job_type: JobType
+    last_status: JobStatusEnum
+    last_message: str
+    last_status_detail: JobStatus | None = None
+    type: Literal["IncompleteJobItemRef"] = "IncompleteJobItemRef"
+
+    def df(self) -> pd.DataFrame:
+        """Present in a pandas DataFrame."""
+        return self.annotations.df().join(
+            pd.DataFrame(
+                {
+                    "project": self.project.annotations.name,
+                    "id": self.id,
+                    "last_status": self.last_status,
+                    "job_item_id": self.job_item_id,
+                    "job_item_integer_id": self.job_item_integer_id,
                 },
                 index=[0],
             )
@@ -523,6 +668,7 @@ Ref = Annotated[
         ProjectRef,
         CircuitRef,
         WasmModuleRef,
+        GpuDecoderConfigRef,
         HUGRRef,
         QIRRef,
         JobRef,
@@ -531,6 +677,8 @@ Ref = Annotated[
         CompilationResultRef,
         ExecutionResultRef,
         CompilationPassRef,
+        SystemRef,
+        IncompleteJobItemRef,
     ],
     Field(discriminator="type"),
 ]
